
// basic variables
var endpoints = new Array();
endpoints["dbpedia"] = "dbpedia.php";
endpoints["eventmedia"] = "eventmedia.php";
endpoints["local"] = "http://localhost:3030/sssw12-ho-s2/query";
endpoints["update"] = "http://localhost:3030/sssw12-ho-s2/update";

var countLocationAttempts     = 0;
var locationAttempsRetryAfter = 500;
var locationAttemptsTimeout    = 10;

// global variables for the object currently considered
var lon = 0.0;
var lat = 0.0;

function init(uri){
    queryForLocation(uri);
    showMessage("title", "This box should show the result of the query in the queryLabel function. If it does not update within a few minutes, something probably\
 got wrong with the code - "+uri);
   showMessage("museums", "This box should show the result of the query in the queryMuseum function. If it does not update within a few minutes, something probably got wrong with the code");
   showMessage("basic_facts", "This box should show the result of the query in the queryBasicFacts function. If it does not update within a few minutes, something probably got wrong with the code");
   showMessage("events", "This box should show the result of the query in the queryEvents function. If it does not update within a few minutes, something probably got wrong with the code");
}

function query(uri){
    checkIfVisited(uri);
    queryLabel(uri);
    queryMuseum(uri);    
    queryEvents(uri);
    queryBasicFacts(uri);
}

// This includes a SPARQL select query that gets the label (?label) of the city bein considered (uri)
//     - the ?desc variable currently also includes the label, change it so that it includes a short description (comment, abstract) of the place instead
//     - How can you guaranty that this description is in english?
function queryLabel(uri){
    var sparqler = new SPARQL.Service(endpoints["dbpedia"]);
    var query = sparqler.createQuery();
    // put the right query here... careful about selecting the language...
    squery = 'select distinct ?label ?desc where {'+
	'<'+uri+'> <http://www.w3.org/2000/01/rdf-schema#label> ?label. '+
	'<'+uri+'> <http://www.w3.org/2000/01/rdf-schema#label> ?desc. '+
	'}';
    // alert("query = "+squery);
    query.query(squery, {failure: showMessage("title", "queryLabel:: query for label of "+uri+" failed"), success: function(json){showLabel(json);}});
}

// This includes a select query with is currently incorrect, and return no result
//    - It should have in the "?link" variable the DBpedia URI of museums located in the considered city ("uri")
//    - It should have in the "?img" variable an image representing the museum (thumbnail).
//    - does your query work in every case? What if there is no image for the museum?
function queryMuseum(uri){
    var sparqler = new SPARQL.Service(endpoints["dbpedia"]);
    var query = sparqler.createQuery();
    squery = 'select distinct ?link ?img where {'+
	'<'+uri+'>'+
	'}';
    // alert("query = "+squery);
    query.query(squery, {failure: showMessage("museums", "queryMuseum:: query for Museum of "+uri+" failed"), success: function(json){showMuseum(json);}});
}

// This currently includes a query that shows all the events nearby 
//    - limit the query to 10 results
//    - show the results that are the most recent first
//    - exclude events of the type http://data.semanticweb.org/ns/swc/ontology#ConferenceEvent
//    - Can you rank according to distance to the considered place instead? 
//    - Can you combine the two criteria date and location in the SPARQL query?
function queryEvents(uri){
    cwa = countLocationAttempts;
    if (lon == 0.0) {
	if (cwa >= locationAttemptsTimeout){
	    alert("Waited "+((countLocationAttempts*locationAttempsRetryAfter)/1000)+" seconds, and I still don't have a location to check the events nearby");
	} else {
	    countLocationAttempts++;
	    setTimeout(function(){queryEvents(uri)}, locationAttempsRetryAfter);
	    return;
	}
    }
    var sparqler = new SPARQL.Service(endpoints["eventmedia"]);
    var query = sparqler.createQuery();
    squery = 'select distinct ?event ?title ?date {'+
	'?event <http://linkedevents.org/ontology/inSpace> ?space .'+
	'?space <http://www.w3.org/2003/01/geo/wgs84_pos#lat> ?lat. '+
	'?space <http://www.w3.org/2003/01/geo/wgs84_pos#long> ?lon.'+
	'?event <http://purl.org/dc/elements/1.1/title> ?title .'+
	'?event <http://linkedevents.org/ontology/atTime> ?odate .'+
	'?odate <http://www.w3.org/2006/time#inXSDDateTime> ?date .'+
	'FILTER ( ( '+
	'( xsd:float(?lat) - '+lat+' ) > 0.0  && ( xsd:float(?lat) - '+lat+' ) < 0.6 '+
	' || ( xsd:float(?lat) - '+lat+' ) < 0.0  && ( xsd:float(?lat) - '+lat+' ) > -0.6 ) '+
	' && ( ( xsd:float(?lon) - '+lon+' ) > 0.0  && ( xsd:float(?lon) - '+lon+' ) < 0.6 '+
	' || ( xsd:float(?lon) - '+lon+' ) < 0.0  && ( xsd:float(?lon) - '+lon+' ) > -0.6 )  '+
        ')'+
	'} limit 10';    
    //    alert("query = "+squery);
    query.query(squery, {failure: showMessage("events", "queryEvents:: query for events near "+uri+" failed"), 
		success: function(json){showEvents(json);}});    
}

// This currently includes a DESCRIBE SPARQL Query, which will show everything associated with the object
// Transform it into a a CONSTRUCT QUERY That 
//    1- Show only literal values
//    2- Select certain interesting properties
//    3- Change the name of the properties into something more readable
function queryBasicFacts(uri){
    var sparqler = new SPARQL.Service(endpoints["dbpedia"]);
    var query = sparqler.createQuery();
    squery = 'describe <'+uri+'>';
    query.query(squery, {failure: showMessage("basic_facts", "queryBasicFacts:: query for basic facts about "+uri+" failed"), success: function(json){showBasicFacts(json, uri);}});
}

// This includes an empty INSERT QUERY. You should transform it so that it records the fact that you visited a place. Note that you need to choose at least of:
//    - a URI for yourself (or your group)
//    - A way to connect your URI to the place that represents the fact that you visited it (it can be very simple)
// To test that you actually inserted the information, you might want to ask FUSEKI to export the RDF contained in the local triple store (./s-get http://localhost:3030/sssw12-ho-s2 default).
function declareVisited(uri){
    squery = 'insert {<'+uri+'>} where {}';
    sendUpdateQuery(endpoints['update'], squery, updateBeenThereImage, [true]);
}

// This includes a DELETE QUERY. You should transform it so that it removes the information that you recorded thanks to the previous INSERT QUERY.
// Again, you can check the effect on the local triple store by exporting the corresponding RDF.
function removeVisited(uri){
    squery = 'delete {<'+uri+'>} where {}';
    sendUpdateQuery(endpoints['update'], squery, updateBeenThereImage, [false]);
}

// This includes an empty ASK query which should check whether it is recorded that you have visited the place, using the representation you have used for the insert query above.
function checkIfVisited(uri){
    var sparqler = new SPARQL.Service(endpoints["local"]);
    var query = sparqler.createQuery();
    squery = 'ASK {<'+uri+'>}';
    query.ask(squery, {failure: function () {updateBeenThereImage("", [false]); }, 
		success: function(bool) {updateBeenThereImage("", [bool]);}});   
}

///////////////////////////////////////////////

function showLabel(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("title", "No results for this URI or wrong query in queryLabel");
	return;
    } 
    elem = document.getElementById("title");
    elem.innerHTML = "<h1>"+json['results']['bindings'][0]['label']['value']+"</h1>";
    elem.innerHTML += '<div class="desc">'+json['results']['bindings'][0]['desc']['value']+"</div>";
}

function showMuseum(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("museums", "No results for this URI or wrong query in queryMuseum");
	return;
    } 
    elem = document.getElementById("museums");
    elem.innerHTML = "<h2>Museums</h2>";
    for (x in json['results']['bindings']){
	elem.innerHTML += '<span class="museum_pic"> <a href="'+json['results']['bindings'][x]['link']['value']+'"><img src="'+json['results']['bindings'][x]['img']['value']+'" /></a></span>';
    }
}

// This includes a select query that retrieves the coordinates of the location of the place
function queryForLocation(uri){
    var sparqler = new SPARQL.Service(endpoints["dbpedia"]);
    var query = sparqler.createQuery();
    // put the right query here... careful about selecting the language... 
    squery = 'select distinct ?lat ?lon where {<'+uri+'> <http://www.w3.org/2003/01/geo/wgs84_pos#lat> ?lat. <'+uri+'> <http://www.w3.org/2003/01/geo/wgs84_pos#long> ?lon} limit 1';
    //    alert("query = "+squery); 
    query.query(squery, {failure: showMessage("events", "queryForLocation:: query for location of "+uri+" failed"), 
    success: function(json){recordLocation(json);}});
}

function recordLocation(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("events", "No results for location of URI or wrong query in queryForLocation");
	return;
    } 
    lat = json['results']['bindings'][0]['lat']['value'];
    lon = json['results']['bindings'][0]['lon']['value'];
}

function showEvents(json){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("events", "No results for this URI or wrong query in queryEvents");
	return;
    } 
    elem = document.getElementById("events");
    elem.innerHTML = "<h2>Recent Events Nearby</h2>";
    for (x in json['results']['bindings']){
	elem.innerHTML += '<span class="event_pic"> <a href="'+json['results']['bindings'][x]['event']['value']+'"><strong>'+json['results']['bindings'][x]['title']['value']+'</strong></a></span>, ';
    }
}

function showBasicFacts(json, uri){
    if (!json || !json['results'] || json['results']['bindings'].length ==0){
	showMessage("basic_facts", "No results for this URI or wrong query in queryBasicFacts");
	return;
    } 
    elem = document.getElementById("basic_facts");
    elem.innerHTML = "<h2>Basic Facts</h2>";
    for (x in json['results']['bindings']){
	if (x > 100) break;
	if (json['results']['bindings'][x]['s']['value'] == uri){	    
	    elem.innerHTML+= '<span class="bfact">'+
		"has "+getFragment(json['results']['bindings'][x]['p']['value'])+" "+getFragment(json['results']['bindings'][x]['o']['value'])+
		'</span><br/>';
	} else if (json['results']['bindings'][x]['o']['value'] == uri){	    
	    elem.innerHTML+= '<span class="bfact">'+
		'is '+getFragment(json['results']['bindings'][x]['p']['value'])+" of "+getFragment(json['results']['bindings'][x]['s']['value'])+
		'</span><br/>';
	    
	}
    }
}

function updateBeenThereImage(res, val){
    elem = document.getElementById("been_there");
    if (val[0] == true){
	elem.innerHTML= '<center>'+
	    '<a href="javascript:removeVisited(\''+uri+'\');"> <img height="120" src="images/been_there_s.png" /></a>'+
	    '</center>';
    } else {
	elem.innerHTML= '<center>'+
	    '<a href="javascript:declareVisited(\''+uri+'\');"> <img height="120" src="images/not_been_there_s.png" /></a>'+
	    '</center>';  
    }
}


